@@ -1,3 +1,5 @@
+use std::convert::TryInto;
+
 pub(crate) mod builder_utils;
 pub(crate) mod deque;
 pub(crate) mod error;
@@ -15,11 +17,7 @@ pub(crate) mod atomic_time;
 
 pub(crate) mod time;
 
-use time::Instant;
-
-pub(crate) trait AccessTime {
-    fn last_accessed(&self) -> Option<Instant>;
-    fn set_last_accessed(&mut self, timestamp: Instant);
-    fn last_modified(&self) -> Option<Instant>;
-    fn set_last_modified(&mut self, timestamp: Instant);
+// Ensures the value fits in a range of `128u32..=u32::MAX`.
+pub(crate) fn sketch_capacity(max_capacity: u64) -> u32 {
+    max_capacity.try_into().unwrap_or(u32::MAX).max(128)
 }
