@@ -1,10 +1,11 @@
-use super::{Cache, SegmentedCache};
+use super::{Cache, SegmentedCache, Weigher};
 use crate::common::builder_utils;
 
 use std::{
     collections::hash_map::RandomState,
     hash::{BuildHasher, Hash},
     marker::PhantomData,
+    sync::Arc,
     time::Duration,
 };
 
@@ -17,11 +18,12 @@ use std::{
 /// # Examples
 ///
 /// ```rust
-/// use moka::sync::CacheBuilder;
-///
+/// use moka::sync::Cache;
 /// use std::time::Duration;
 ///
-/// let cache = CacheBuilder::new(10_000) // Max 10,000 elements
+/// let cache = Cache::builder()
+///     // Max 10,000 entries
+///     .max_capacity(10_000)
 ///     // Time to live (TTL): 30 minutes
 ///     .time_to_live(Duration::from_secs(30 * 60))
 ///     // Time to idle (TTI):  5 minutes
@@ -39,32 +41,47 @@ use std::{
 /// // after 30 minutes (TTL) from the insert().
 /// ```
 ///
-pub struct CacheBuilder<C> {
-    max_capacity: usize,
+pub struct CacheBuilder<K, V, C> {
+    max_capacity: Option<u64>,
     initial_capacity: Option<usize>,
     num_segments: Option<usize>,
+    weigher: Option<Weigher<K, V>>,
     time_to_live: Option<Duration>,
     time_to_idle: Option<Duration>,
     invalidator_enabled: bool,
     cache_type: PhantomData<C>,
 }
 
-impl<K, V> CacheBuilder<Cache<K, V, RandomState>>
+impl<K, V> Default for CacheBuilder<K, V, Cache<K, V, RandomState>>
+where
+    K: Eq + Hash + Send + Sync + 'static,
+    V: Clone + Send + Sync + 'static,
+{
+    fn default() -> Self {
+        Self {
+            max_capacity: None,
+            initial_capacity: None,
+            num_segments: None,
+            weigher: None,
+            time_to_live: None,
+            time_to_idle: None,
+            invalidator_enabled: false,
+            cache_type: Default::default(),
+        }
+    }
+}
+
+impl<K, V> CacheBuilder<K, V, Cache<K, V, RandomState>>
 where
     K: Eq + Hash + Send + Sync + 'static,
     V: Clone + Send + Sync + 'static,
 {
     /// Construct a new `CacheBuilder` that will be used to build a `Cache` or
     /// `SegmentedCache` holding up to `max_capacity` entries.
-    pub fn new(max_capacity: usize) -> Self {
+    pub fn new(max_capacity: u64) -> Self {
         Self {
-            max_capacity,
-            initial_capacity: None,
-            num_segments: None,
-            time_to_live: None,
-            time_to_idle: None,
-            invalidator_enabled: false,
-            cache_type: PhantomData::default(),
+            max_capacity: Some(max_capacity),
+            ..Default::default()
         }
     }
 
@@ -72,14 +89,18 @@ where
     ///
     /// # Panics
     ///
-    /// Panics if `num_segments` is less than or equals to 1.
-    pub fn segments(self, num_segments: usize) -> CacheBuilder<SegmentedCache<K, V, RandomState>> {
-        assert!(num_segments > 1);
+    /// Panics if `num_segments` is zero.
+    pub fn segments(
+        self,
+        num_segments: usize,
+    ) -> CacheBuilder<K, V, SegmentedCache<K, V, RandomState>> {
+        assert!(num_segments != 0);
 
         CacheBuilder {
             max_capacity: self.max_capacity,
             initial_capacity: self.initial_capacity,
             num_segments: Some(num_segments),
+            weigher: None,
             time_to_live: self.time_to_live,
             time_to_idle: self.time_to_idle,
             invalidator_enabled: self.invalidator_enabled,
@@ -104,6 +125,7 @@ where
             self.max_capacity,
             self.initial_capacity,
             build_hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
             self.invalidator_enabled,
@@ -129,6 +151,7 @@ where
             self.max_capacity,
             self.initial_capacity,
             hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
             self.invalidator_enabled,
@@ -136,7 +159,7 @@ where
     }
 }
 
-impl<K, V> CacheBuilder<SegmentedCache<K, V, RandomState>>
+impl<K, V> CacheBuilder<K, V, SegmentedCache<K, V, RandomState>>
 where
     K: Eq + Hash + Send + Sync + 'static,
     V: Clone + Send + Sync + 'static,
@@ -159,6 +182,7 @@ where
             self.initial_capacity,
             self.num_segments.unwrap(),
             build_hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
             self.invalidator_enabled,
@@ -185,6 +209,7 @@ where
             self.initial_capacity,
             self.num_segments.unwrap(),
             hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
             self.invalidator_enabled,
@@ -192,11 +217,30 @@ where
     }
 }
 
-impl<C> CacheBuilder<C> {
-    /// Sets the initial capacity of the cache.
-    pub fn initial_capacity(self, capacity: usize) -> Self {
+impl<K, V, C> CacheBuilder<K, V, C> {
+    /// Sets the max capacity of the cache.
+    pub fn max_capacity(self, max_capacity: u64) -> Self {
         Self {
-            initial_capacity: Some(capacity),
+            max_capacity: Some(max_capacity),
+            ..self
+        }
+    }
+
+    /// Sets the initial capacity (number of entries) of the cache.
+    pub fn initial_capacity(self, number_of_entries: usize) -> Self {
+        Self {
+            initial_capacity: Some(number_of_entries),
+            ..self
+        }
+    }
+
+    /// Sets the weigher closure of the cache.
+    ///
+    /// The closure should take `&K` and `&V` as the arguments and returns a `u32`
+    /// representing the relative size of the entry.
+    pub fn weigher(self, weigher: impl Fn(&K, &V) -> u32 + Send + Sync + 'static) -> Self {
+        Self {
+            weigher: Some(Arc::new(weigher)),
             ..self
         }
     }
@@ -252,7 +296,6 @@ impl<C> CacheBuilder<C> {
 
 #[cfg(test)]
 mod tests {
-    use super::Cache;
     use super::CacheBuilder;
 
     use std::time::Duration;
@@ -262,7 +305,7 @@ mod tests {
         // Cache<char, String>
         let cache = CacheBuilder::new(100).build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), None);
         assert_eq!(cache.time_to_idle(), None);
         assert_eq!(cache.num_segments(), 1);
@@ -275,7 +318,7 @@ mod tests {
             .time_to_idle(Duration::from_secs(15 * 60))
             .build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), Some(Duration::from_secs(45 * 60)));
         assert_eq!(cache.time_to_idle(), Some(Duration::from_secs(15 * 60)));
         assert_eq!(cache.num_segments(), 1);
@@ -289,7 +332,7 @@ mod tests {
         // SegmentCache<char, String>
         let cache = CacheBuilder::new(100).segments(16).build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), None);
         assert_eq!(cache.time_to_idle(), None);
         assert_eq!(cache.num_segments(), 16_usize.next_power_of_two());
@@ -303,7 +346,7 @@ mod tests {
             .time_to_idle(Duration::from_secs(15 * 60))
             .build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), Some(Duration::from_secs(45 * 60)));
         assert_eq!(cache.time_to_idle(), Some(Duration::from_secs(15 * 60)));
         assert_eq!(cache.num_segments(), 16_usize.next_power_of_two());
@@ -316,7 +359,7 @@ mod tests {
     #[should_panic(expected = "time_to_live is longer than 1000 years")]
     fn build_cache_too_long_ttl() {
         let thousand_years_secs: u64 = 1000 * 365 * 24 * 3600;
-        let builder: CacheBuilder<Cache<char, String>> = CacheBuilder::new(100);
+        let builder: CacheBuilder<char, String, _> = CacheBuilder::new(100);
         let duration = Duration::from_secs(thousand_years_secs);
         builder
             .time_to_live(duration + Duration::from_secs(1))
@@ -327,7 +370,7 @@ mod tests {
     #[should_panic(expected = "time_to_idle is longer than 1000 years")]
     fn build_cache_too_long_tti() {
         let thousand_years_secs: u64 = 1000 * 365 * 24 * 3600;
-        let builder: CacheBuilder<Cache<char, String>> = CacheBuilder::new(100);
+        let builder: CacheBuilder<char, String, _> = CacheBuilder::new(100);
         let duration = Duration::from_secs(thousand_years_secs);
         builder
             .time_to_idle(duration + Duration::from_secs(1))
