@@ -1,4 +1,4 @@
-use super::{cache::Cache, ConcurrentCacheExt};
+use super::{cache::Cache, CacheBuilder, ConcurrentCacheExt, Weigher};
 use crate::PredicateError;
 
 use std::{
@@ -60,7 +60,7 @@ where
     V: Clone + Send + Sync + 'static,
 {
     /// Constructs a new `SegmentedCache<K, V>` that has multiple internal
-    /// segments and will store up to the `max_capacity` entries.
+    /// segments and will store up to the `max_capacity`.
     ///
     /// To adjust various configuration knobs such as `initial_capacity` or
     /// `time_to_live`, use the [`CacheBuilder`][builder-struct].
@@ -70,18 +70,27 @@ where
     /// # Panics
     ///
     /// Panics if `num_segments` is 0.
-    pub fn new(max_capacity: usize, num_segments: usize) -> Self {
+    pub fn new(max_capacity: u64, num_segments: usize) -> Self {
         let build_hasher = RandomState::default();
         Self::with_everything(
-            max_capacity,
+            Some(max_capacity),
             None,
             num_segments,
             build_hasher,
             None,
             None,
+            None,
             false,
         )
     }
+
+    /// Returns a [`CacheBuilder`][builder-struct], which can builds a
+    /// `SegmentedCache` with various configuration knobs.
+    ///
+    /// [builder-struct]: ./struct.CacheBuilder.html
+    pub fn builder(num_segments: usize) -> CacheBuilder<K, V, SegmentedCache<K, V, RandomState>> {
+        CacheBuilder::default().segments(num_segments)
+    }
 }
 
 impl<K, V, S> SegmentedCache<K, V, S>
@@ -93,11 +102,13 @@ where
     /// # Panics
     ///
     /// Panics if `num_segments` is 0.
+    #[allow(clippy::too_many_arguments)]
     pub(crate) fn with_everything(
-        max_capacity: usize,
+        max_capacity: Option<u64>,
         initial_capacity: Option<usize>,
         num_segments: usize,
         build_hasher: S,
+        weigher: Option<Weigher<K, V>>,
         time_to_live: Option<Duration>,
         time_to_idle: Option<Duration>,
         invalidator_enabled: bool,
@@ -108,6 +119,7 @@ where
                 initial_capacity,
                 num_segments,
                 build_hasher,
+                weigher,
                 time_to_live,
                 time_to_idle,
                 invalidator_enabled,
@@ -244,7 +256,7 @@ where
     }
 
     /// Returns the `max_capacity` of this cache.
-    pub fn max_capacity(&self) -> usize {
+    pub fn max_capacity(&self) -> Option<u64> {
         self.inner.desired_capacity
     }
 
@@ -263,6 +275,24 @@ where
         self.inner.segments.len()
     }
 
+    #[cfg(test)]
+    fn estimated_entry_count(&self) -> u64 {
+        self.inner
+            .segments
+            .iter()
+            .map(|seg| seg.estimated_entry_count())
+            .sum()
+    }
+
+    #[cfg(test)]
+    fn weighted_size(&self) -> u64 {
+        self.inner
+            .segments
+            .iter()
+            .map(|seg| seg.weighted_size())
+            .sum()
+    }
+
     // /// This is used by unit tests to get consistent result.
     // #[cfg(test)]
     // pub(crate) fn reconfigure_for_testing(&mut self) {
@@ -291,13 +321,9 @@ where
 impl<K, V, S> SegmentedCache<K, V, S>
 where
     K: Hash + Eq + Send + Sync + 'static,
-    V: Send + Sync + 'static,
+    V: Clone + Send + Sync + 'static,
     S: BuildHasher + Clone + Send + Sync + 'static,
 {
-    fn table_size(&self) -> usize {
-        self.inner.segments.iter().map(|seg| seg.table_size()).sum()
-    }
-
     fn invalidation_predicate_count(&self) -> usize {
         self.inner
             .segments
@@ -345,7 +371,7 @@ impl MockExpirationClock {
 }
 
 struct Inner<K, V, S> {
-    desired_capacity: usize,
+    desired_capacity: Option<u64>,
     segments: Box<[Cache<K, V, S>]>,
     build_hasher: S,
     segment_shift: u32,
@@ -360,11 +386,13 @@ where
     /// # Panics
     ///
     /// Panics if `num_segments` is 0.
+    #[allow(clippy::too_many_arguments)]
     fn new(
-        max_capacity: usize,
+        max_capacity: Option<u64>,
         initial_capacity: Option<usize>,
         num_segments: usize,
         build_hasher: S,
+        weigher: Option<Weigher<K, V>>,
         time_to_live: Option<Duration>,
         time_to_idle: Option<Duration>,
         invalidator_enabled: bool,
@@ -374,16 +402,17 @@ where
         let actual_num_segments = num_segments.next_power_of_two();
         let segment_shift = 64 - actual_num_segments.trailing_zeros();
         // TODO: Round up.
-        let seg_capacity = max_capacity / actual_num_segments;
+        let seg_max_capacity = max_capacity.map(|n| n / actual_num_segments as u64);
         let seg_init_capacity = initial_capacity.map(|cap| cap / actual_num_segments);
         // NOTE: We cannot initialize the segments as `vec![cache; actual_num_segments]`
         // because Cache::clone() does not clone its inner but shares the same inner.
         let segments = (0..num_segments)
             .map(|_| {
                 Cache::with_everything(
-                    seg_capacity,
+                    seg_max_capacity,
                     seg_init_capacity,
                     build_hasher.clone(),
+                    weigher.as_ref().map(Arc::clone),
                     time_to_live,
                     time_to_idle,
                     invalidator_enabled,
@@ -467,7 +496,7 @@ mod tests {
         assert_eq!(cache.get(&"d"), None); //   d -> 2
 
         // "d" should be admitted and "c" should be evicted
-        // because d's frequency is higher then c's.
+        // because d's frequency is higher than c's.
         cache.insert("d", "dennis");
         cache.sync();
         assert_eq!(cache.get(&"a"), Some("alice"));
@@ -478,6 +507,89 @@ mod tests {
         cache.invalidate(&"b");
     }
 
+    #[test]
+    fn size_aware_eviction() {
+        let weigher = |_k: &&str, v: &(&str, u32)| v.1;
+
+        let alice = ("alice", 10);
+        let bob = ("bob", 15);
+        let bill = ("bill", 20);
+        let cindy = ("cindy", 5);
+        let david = ("david", 15);
+        let dennis = ("dennis", 15);
+
+        let mut cache = SegmentedCache::builder(1)
+            .max_capacity(31)
+            .weigher(weigher)
+            .build();
+        cache.reconfigure_for_testing();
+
+        // Make the cache exterior immutable.
+        let cache = cache;
+
+        cache.insert("a", alice);
+        cache.insert("b", bob);
+        assert_eq!(cache.get(&"a"), Some(alice));
+        assert_eq!(cache.get(&"b"), Some(bob));
+        cache.sync();
+        // order (LRU -> MRU) and counts: a -> 1, b -> 1
+
+        cache.insert("c", cindy);
+        assert_eq!(cache.get(&"c"), Some(cindy));
+        // order and counts: a -> 1, b -> 1, c -> 1
+        cache.sync();
+
+        assert_eq!(cache.get(&"a"), Some(alice));
+        assert_eq!(cache.get(&"b"), Some(bob));
+        cache.sync();
+        // order and counts: c -> 1, a -> 2, b -> 2
+
+        // To enter "d" (weight: 15), it needs to evict "c" (w: 5) and "a" (w: 10).
+        // "d" must have higher count than 3, which is the aggregated count
+        // of "a" and "c".
+        cache.insert("d", david); //   count: d -> 0
+        cache.sync();
+        assert_eq!(cache.get(&"d"), None); //   d -> 1
+
+        cache.insert("d", david);
+        cache.sync();
+        assert_eq!(cache.get(&"d"), None); //   d -> 2
+
+        cache.insert("d", david);
+        cache.sync();
+        assert_eq!(cache.get(&"d"), None); //   d -> 3
+
+        cache.insert("d", david);
+        cache.sync();
+        assert_eq!(cache.get(&"d"), None); //   d -> 4
+
+        // Finally "d" should be admitted by evicting "c" and "a".
+        cache.insert("d", dennis);
+        cache.sync();
+        assert_eq!(cache.get(&"a"), None);
+        assert_eq!(cache.get(&"b"), Some(bob));
+        assert_eq!(cache.get(&"c"), None);
+        assert_eq!(cache.get(&"d"), Some(dennis));
+
+        // Update "b" with "bill" (w: 15 -> 20). This should evict "d" (w: 15).
+        cache.insert("b", bill);
+        cache.sync();
+        assert_eq!(cache.get(&"b"), Some(bill));
+        assert_eq!(cache.get(&"d"), None);
+
+        // Re-add "a" (w: 10) and update "b" with "bob" (w: 20 -> 15).
+        cache.insert("a", alice);
+        cache.insert("b", bob);
+        cache.sync();
+        assert_eq!(cache.get(&"a"), Some(alice));
+        assert_eq!(cache.get(&"b"), Some(bob));
+        assert_eq!(cache.get(&"d"), None);
+
+        // Verify the sizes.
+        assert_eq!(cache.estimated_entry_count(), 2);
+        assert_eq!(cache.weighted_size(), 25);
+    }
+
     #[test]
     fn basic_multi_threads() {
         let num_threads = 4;
@@ -584,7 +696,7 @@ mod tests {
         assert_eq!(cache.get(&1), Some("bob"));
         // This should survive as it was inserted after calling invalidate_entries_if.
         assert_eq!(cache.get(&3), Some("alice"));
-        assert_eq!(cache.table_size(), 2);
+        assert_eq!(cache.estimated_entry_count(), 2);
         assert_eq!(cache.invalidation_predicate_count(), SEGMENTS * 0);
 
         mock.increment(Duration::from_secs(5)); // 15 secs from the start.
@@ -601,7 +713,7 @@ mod tests {
 
         assert!(cache.get(&1).is_none());
         assert!(cache.get(&3).is_none());
-        assert_eq!(cache.table_size(), 0);
+        assert_eq!(cache.estimated_entry_count(), 0);
         assert_eq!(cache.invalidation_predicate_count(), SEGMENTS * 0);
 
         Ok(())
