@@ -1,4 +1,4 @@
-use super::Cache;
+use super::{Cache, Weigher};
 use crate::common::builder_utils;
 
 use std::{
@@ -15,11 +15,12 @@ use std::{
 /// # Examples
 ///
 /// ```rust
-/// use moka::unsync::CacheBuilder;
-///
+/// use moka::unsync::Cache;
 /// use std::time::Duration;
 ///
-/// let mut cache = CacheBuilder::new(10_000) // Max 10,000 elements
+/// let mut cache = Cache::builder()
+///     // Max 10,000 elements
+///     .max_capacity(10_000)
 ///     // Time to live (TTL): 30 minutes
 ///     .time_to_live(Duration::from_secs(30 * 60))
 ///     // Time to idle (TTI):  5 minutes
@@ -37,27 +38,41 @@ use std::{
 /// // after 30 minutes (TTL) from the insert().
 /// ```
 ///
-pub struct CacheBuilder<C> {
-    max_capacity: usize,
+pub struct CacheBuilder<K, V, C> {
+    max_capacity: Option<u64>,
     initial_capacity: Option<usize>,
+    weigher: Option<Weigher<K, V>>,
     time_to_live: Option<Duration>,
     time_to_idle: Option<Duration>,
     cache_type: PhantomData<C>,
 }
 
-impl<K, V> CacheBuilder<Cache<K, V, RandomState>>
+impl<K, V> Default for CacheBuilder<K, V, Cache<K, V, RandomState>>
+where
+    K: Eq + Hash,
+{
+    fn default() -> Self {
+        Self {
+            max_capacity: None,
+            initial_capacity: None,
+            weigher: None,
+            time_to_live: None,
+            time_to_idle: None,
+            cache_type: Default::default(),
+        }
+    }
+}
+
+impl<K, V> CacheBuilder<K, V, Cache<K, V, RandomState>>
 where
     K: Eq + Hash,
 {
     /// Construct a new `CacheBuilder` that will be used to build a `Cache` holding
     /// up to `max_capacity` entries.
-    pub fn new(max_capacity: usize) -> Self {
+    pub fn new(max_capacity: u64) -> Self {
         Self {
-            max_capacity,
-            initial_capacity: None,
-            time_to_live: None,
-            time_to_idle: None,
-            cache_type: PhantomData::default(),
+            max_capacity: Some(max_capacity),
+            ..Default::default()
         }
     }
 
@@ -75,6 +90,7 @@ where
             self.max_capacity,
             self.initial_capacity,
             build_hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
         )
@@ -96,17 +112,37 @@ where
             self.max_capacity,
             self.initial_capacity,
             hasher,
+            self.weigher,
             self.time_to_live,
             self.time_to_idle,
         )
     }
 }
 
-impl<C> CacheBuilder<C> {
-    /// Sets the initial capacity of the cache.
-    pub fn initial_capacity(self, capacity: usize) -> Self {
+impl<K, V, C> CacheBuilder<K, V, C> {
+    /// Sets the max capacity of the cache.
+    pub fn max_capacity(self, max_capacity: u64) -> Self {
         Self {
-            initial_capacity: Some(capacity),
+            max_capacity: Some(max_capacity),
+            ..self
+        }
+    }
+
+    /// Sets the initial capacity (number of entries) of the cache.
+    pub fn initial_capacity(self, number_of_entries: usize) -> Self {
+        Self {
+            initial_capacity: Some(number_of_entries),
+            ..self
+        }
+    }
+
+    /// Sets the weigher closure of the cache.
+    ///
+    /// The closure should take `&K` and `&V` as the arguments and returns a `u32`
+    /// representing the relative size of the entry.
+    pub fn weigher(self, weigher: impl FnMut(&K, &V) -> u32 + 'static) -> Self {
+        Self {
+            weigher: Some(Box::new(weigher)),
             ..self
         }
     }
@@ -148,7 +184,6 @@ impl<C> CacheBuilder<C> {
 
 #[cfg(test)]
 mod tests {
-    use super::Cache;
     use super::CacheBuilder;
 
     use std::time::Duration;
@@ -158,7 +193,7 @@ mod tests {
         // Cache<char, String>
         let mut cache = CacheBuilder::new(100).build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), None);
         assert_eq!(cache.time_to_idle(), None);
 
@@ -170,7 +205,7 @@ mod tests {
             .time_to_idle(Duration::from_secs(15 * 60))
             .build();
 
-        assert_eq!(cache.max_capacity(), 100);
+        assert_eq!(cache.max_capacity(), Some(100));
         assert_eq!(cache.time_to_live(), Some(Duration::from_secs(45 * 60)));
         assert_eq!(cache.time_to_idle(), Some(Duration::from_secs(15 * 60)));
 
@@ -182,7 +217,7 @@ mod tests {
     #[should_panic(expected = "time_to_live is longer than 1000 years")]
     fn build_cache_too_long_ttl() {
         let thousand_years_secs: u64 = 1000 * 365 * 24 * 3600;
-        let builder: CacheBuilder<Cache<char, String>> = CacheBuilder::new(100);
+        let builder: CacheBuilder<char, String, _> = CacheBuilder::new(100);
         let duration = Duration::from_secs(thousand_years_secs);
         builder
             .time_to_live(duration + Duration::from_secs(1))
@@ -193,7 +228,7 @@ mod tests {
     #[should_panic(expected = "time_to_idle is longer than 1000 years")]
     fn build_cache_too_long_tti() {
         let thousand_years_secs: u64 = 1000 * 365 * 24 * 3600;
-        let builder: CacheBuilder<Cache<char, String>> = CacheBuilder::new(100);
+        let builder: CacheBuilder<char, String, _> = CacheBuilder::new(100);
         let duration = Duration::from_secs(thousand_years_secs);
         builder
             .time_to_idle(duration + Duration::from_secs(1))
